// DicksBackupDoc.cpp : implementation of the CDicksBackupDoc class
//

#include "stdafx.h"
#include "MyIncludes.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDicksBackupDoc

IMPLEMENT_DYNCREATE(CDicksBackupDoc, CDocument)

BEGIN_MESSAGE_MAP(CDicksBackupDoc, CDocument)
	//{{AFX_MSG_MAP(CDicksBackupDoc)
	ON_COMMAND(ID_EDIT_ADD_FILES, OnEditAddFiles)
	ON_COMMAND(ID_MY_FILE_SAVE_AS, OnMyFileSaveAs)
	ON_COMMAND(ID_MY_FILE_SAVE, OnMyFileSave)
	ON_COMMAND(ID_MY_FILE_OPEN, OnMyFileOpen)
	ON_UPDATE_COMMAND_UI(ID_MY_FILE_SAVE, OnUpdateMyFileSave)
	ON_COMMAND(ID_EDIT_DO_BACKUP, OnDoBackup)
	ON_UPDATE_COMMAND_UI(ID_EDIT_DO_BACKUP, OnUpdateDoBackup)
	ON_COMMAND(ID_EDIT_REFRESH, OnEditRefresh)
	ON_UPDATE_COMMAND_UI(ID_EDIT_REFRESH, OnUpdateEditRefresh)
	ON_COMMAND(ID_MY_FILE_CREATE, OnMyFileCreate)
	ON_COMMAND(ID_MY_FILE_CLOSE, OnMyFileClose)
	ON_UPDATE_COMMAND_UI(ID_MY_FILE_CLOSE, OnUpdateMyFileClose)
	ON_UPDATE_COMMAND_UI(ID_EDIT_ADD_FILES, OnUpdateEditAddFiles)
	ON_COMMAND(ID_EDIT_BACKUP_SEL, OnEditBackupSel)
	ON_UPDATE_COMMAND_UI(ID_EDIT_BACKUP_SEL, OnUpdateEditBackupSel)
	ON_COMMAND(ID_EDIT_APPEND, OnEditAppend)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDicksBackupDoc construction/destruction

CDicksBackupDoc::CDicksBackupDoc()
{
	CDicksBackupApp * pApp = (CDicksBackupApp *)AfxGetApp();
	m_addFilesDirStr = pApp->GetProfileString(settings, addDirStr, _T(".\\"));
	m_backupInitDirStr = pApp->GetProfileString(settings, backupDirStr, _T(".\\"));
	m_lastOpenedFileStr = ("");
	m_bBackupSelected = FALSE;
	m_bSilent = FALSE;
}

CDicksBackupDoc::~CDicksBackupDoc()
{
	CDicksBackupApp * pApp = (CDicksBackupApp *)AfxGetApp();
	pApp->WriteProfileString(settings, addDirStr, m_addFilesDirStr);
	pApp->WriteProfileString(settings, backupDirStr, m_backupInitDirStr);

	ClearItemData();
}

/////////////////////////////////////////////////////////////////////////////
// CDicksBackupDoc serializatio
void CDicksBackupDoc::Serialize(CArchive& ar)
{
	UINT nCount = 0;
	CItemData * pItemData = NULL;

	if(ar.IsStoring()) // storing backup manager data
	{
		CDicksBackupView * pEditView = (CDicksBackupView*)m_viewList.GetHead();
		nCount = pEditView->GetRows();
		ASSERT(nCount == (UINT)m_itemDataPtrArray.GetSize());

		// display progress in status bar
		PROGRESS(nCount, 1, _T(" Saving manager files"));

		CString verStr(versionString);
		ar << verStr; // ID for this kind of backup file
		ar << nCount; // count of triplits of strings

		// store groups of 3-strings strings (source path, archive path, and notes)
		for(UINT i = 0; i < nCount; i++)
		{
			pItemData = (CItemData *)pEditView->m_pListCtrl->GetItemData(i);

			ar << pItemData->m_sourcePathStr;
			ar << pItemData->m_archivePathStr;
			ar << pItemData->m_notesStr; 

			aProgerssCtrl.StepIt();
		}
		// ensure ready message in status bar
		READY_MSG();
	}
	else if(AfxGetMainWnd() != NULL)  // retreiving backup manager data
	{
		ClearItemData();
		CString sourcePathStr, archivePathStr, notesStr, verStr; 

		ar >> verStr;

		if(verStr.Compare(versionString) == 0)
		{
			ar >> nCount; // get count of string triplits  

			// display progress in status bar
			PROGRESS(nCount, 1, _T(" Locating Files, Please Wait."));

			// retrieve triplit strings source path, archive path, and notes
			for(UINT i = 0; i < nCount; i++)
			{
				ar >> sourcePathStr;
				ar >> archivePathStr;
				ar >> notesStr;

				pItemData = new CItemData(sourcePathStr, archivePathStr, notesStr);
				m_itemDataPtrArray.Add(pItemData);

				aProgerssCtrl.StepIt();
			}
		}
		else
		{
			AfxGetMainWnd()->MessageBox(_T("This is not a Backup file!"), myOpenBackupFileTitle, MB_ICONERROR);
		}

		READY_MSG();
	}
}

/////////////////////////////////////////////////////////////////////////////
// CDicksBackupDoc diagnostics

#ifdef _DEBUG
void CDicksBackupDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CDicksBackupDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CDicksBackupDoc commands

void CDicksBackupDoc::ClearItemData()
{
	UINT nSize = m_itemDataPtrArray.GetSize();

	for(UINT i = 0; i < nSize; i++)
	{
		delete m_itemDataPtrArray[i];
	}

	m_itemDataPtrArray.RemoveAll();
}

void CDicksBackupDoc::OnEditAddFiles() 
{
	char * buf = new char[BYTES_IN_MULTI_FILE_PATN_NAME_BUFFER];

	if(buf)
	{
		// pointer to view class
		CDicksBackupView * pEditView = (CDicksBackupView*)m_viewList.GetHead();
		// remove all selections
		pEditView->SendMessage(WM_COMMAND, ID_EDIT_SELECT_NONE);
		// get even files with shareing violation, but directories not included
		CFileDialog fDlg(TRUE, NULL, NULL, myOpenFlags | OFN_SHAREAWARE, myAllFileFilter, NULL);
			
		fDlg.m_ofn.lpstrFile = buf;
		fDlg.m_ofn.nMaxFile = BYTES_IN_MULTI_FILE_PATN_NAME_BUFFER;
		// buffer for multi file paths (only need to zero first element)
		buf[0] = '\0';
		fDlg.m_ofn.lpstrInitialDir = m_addFilesDirStr;
		fDlg.m_ofn.lpstrTitle = myOpenAllFileTitle;

		if(IDOK == fDlg.DoModal())
		{
			CString sourcePathStr;
			POSITION aPos = fDlg.GetStartPosition();
			
			if(aPos)
			{
				// get each file selected
				while(aPos)
				{
					sourcePathStr = fDlg.GetNextPathName(aPos);
					AddFileToBackup(sourcePathStr);
				}
				// parse last file for current directory
				m_addFilesDirStr = sourcePathStr.Left(sourcePathStr.ReverseFind(_T('\\')) + 1);
			}
		}
		else
		{
			DWORD errVal = CommDlgExtendedError();

			if(errVal)
			{
				CString aStr;
				aStr.Format(_T("File open dialog error\nError code = 0X%X"), errVal);
				(CMainFrame *)AfxGetMainWnd()->MessageBox(aStr, myOpenAllFileTitle, MB_ICONSTOP);
			}
		}

		delete []buf;
	}
	else
	{
		 (CMainFrame *)AfxGetMainWnd()->MessageBox(_T("Memory allocation error"), myOpenAllFileTitle, MB_ICONSTOP);
	}
}

void CDicksBackupDoc::OnUpdateEditAddFiles(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!GetPathName().IsEmpty());	
}

// We're here because of dragging files from explorer to this program
// or a call from OnEditAddFiles()
void CDicksBackupDoc::AddFileToBackup(CString &pathStr)
{
	WIN32_FIND_DATA fd;
	// file must exists on disk and not be a directory
	if(INVALID_HANDLE_VALUE != FindFirstFile(pathStr, &fd) &&
	   0 == (fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
	{
		UINT nItems = m_itemDataPtrArray.GetSize();
		// check if file already is in the entire list
		for(UINT i = 0; i < nItems; i++)
		{
			if(pathStr.CompareNoCase(m_itemDataPtrArray[i]->m_sourcePathStr) == 0)
			{
				return;
			}
		}

		// check if the same named file is already in the archive directory
		CString archiveDirStr = GetPathName().Left(GetPathName().ReverseFind(_T('\\')) + 1);

		if(!archiveDirStr.IsEmpty())
		{
			char fileNameStr[_MAX_FNAME], extStr[_MAX_EXT ];
			_splitpath(pathStr, NULL, NULL, fileNameStr, extStr);

			CString archivePartialPathStr = archiveDirStr + fileNameStr;
			CString archivePathStr = archivePartialPathStr + extStr;

			// try to find a different name if file already exists in directory
			for(int i = 1; i < MAX_TRIES_TO_FIND_ARCHIVE_FILE_NAME; i++)
			{
				if(INVALID_HANDLE_VALUE == FindFirstFile(archivePathStr, &fd))
				{
					// found a usuable name
					CFile aFile;
					//create zero length file in archive directory
					if(aFile.Open(archivePathStr, CFile::modeCreate))
					{
						aFile.Close();

						CItemData * pItemData = new CItemData(pathStr, archivePathStr);
						m_itemDataPtrArray.Add(pItemData);

						((CDicksBackupView*)m_viewList.GetHead())->AppendItemToList(pItemData);
						SetModifiedFlag(TRUE); 
					}

					return;
				}

				// try another file name
				archivePathStr.Format(_T("%s[%d]%s"), archivePartialPathStr, i, extStr);
			}

			if(i >= MAX_TRIES_TO_FIND_ARCHIVE_FILE_NAME)
			{
				CString aStr;
				aStr.Format(_T("Can't find a substitute archive\nfile name after %d tries!"), 
							MAX_TRIES_TO_FIND_ARCHIVE_FILE_NAME);

				(CMainFrame *)AfxGetMainWnd()->MessageBox(aStr, mySaveBackupFileTitle, MB_ICONWARNING);
			}
		}
		else
		{
			(CMainFrame *)AfxGetMainWnd()->MessageBox(_T("Open or create a New backup manager first!"), 
													  mySaveBackupFileTitle, MB_ICONWARNING);
		}
	}
}

// add '*' to title when document is modified
void CDicksBackupDoc::SetModifiedFlag(BOOL bModified)
{
	if(bModified && !IsModified())
	{
		SetTitle(GetTitle() + strIndicator);
	}

	if(IsModified() && !bModified) // remove "*" to view's title bar
	{
		CString strTitle = GetTitle();
		SetTitle(strTitle.Left(strTitle.GetLength() - (sizeof(strIndicator) - 1)));
	}

	UpdateFrameCounts();
	CDocument::SetModifiedFlag(bModified);
}

void CDicksBackupDoc::OnMyFileSaveAs() 
{
	CString pathName = GetPathName();
	CString fileName = pathName.Mid(pathName.ReverseFind(_T('\\')) + 1);

	CFileDialog fDlg(FALSE, NULL, fileName, mySaveAsFlags, myBackupFilter, NULL);
		
	fDlg.m_ofn.lpstrInitialDir = m_backupInitDirStr;
	fDlg.m_ofn.lpstrTitle = mySaveBackupFileTitle;

	if(IDOK == fDlg.DoModal())
	{
		CString fileName = fDlg.GetFileName();
		CString filePath = fDlg.GetPathName();

		if(fileName.ReverseFind('.') < 0) // extension ?
		{
			// add default extension
			fileName += dfltExtension;
			filePath += dfltExtension;
		}

		m_backupInitDirStr = filePath.Left(filePath.GetLength() - fileName.GetLength());

		OnSaveDocument(filePath); // saves with a new name
		SetPathName(filePath, TRUE);
		CDicksBackupView * pEditView = (CDicksBackupView*)m_viewList.GetHead();
		pEditView->OnInitialUpdate();
	}
	else
	{
		DWORD errVal = CommDlgExtendedError();

		if(errVal)
		{
			CString aStr;
			aStr.Format(_T("File save dialog error\nError code = 0X%X"), errVal);
			(CMainFrame *)AfxGetMainWnd()->MessageBox(aStr, mySaveBackupFileTitle, MB_ICONSTOP);
		}
	}
}

void CDicksBackupDoc::OnMyFileSave() 
{
	CString filePath = GetPathName();
	CString fileName = filePath.Mid(filePath.ReverseFind(_T('\\')) + 1);

	if(fileName.IsEmpty()) // an untitled new file
	{
		CFileDialog fDlg(FALSE, NULL, "untitled", mySaveAsFlags, myBackupFilter, NULL);
			
		fDlg.m_ofn.lpstrInitialDir = m_backupInitDirStr;
		fDlg.m_ofn.lpstrTitle = mySaveBackupFileTitle;

		if(IDOK == fDlg.DoModal())
		{
			filePath = fDlg.GetPathName();
			fileName = fDlg.GetFileName();
			m_backupInitDirStr = filePath.Left(filePath.GetLength() - fileName.GetLength());
			OnSaveDocument(filePath); // saves untitled with new name
			SetPathName(filePath, TRUE);
		}
		else
		{
			DWORD errVal = CommDlgExtendedError();

			if(errVal)
			{
				CString aStr;
				aStr.Format(_T("File save dialog error\nError code = 0X%X"), errVal);
				(CMainFrame *)AfxGetMainWnd()->MessageBox(aStr, mySaveBackupFileTitle, MB_ICONSTOP);
			}
		}
	}
	else
	{
		OnSaveDocument(filePath); // normal save runs silent
		SetPathName(filePath, TRUE);
	}
}

void CDicksBackupDoc::OnUpdateMyFileSave(CCmdUI* pCmdUI) 
{
	CDicksBackupView * pEditView = (CDicksBackupView*)m_viewList.GetHead();
	pCmdUI->Enable(pEditView->GetRows() && IsModified());
}

void CDicksBackupDoc::OnMyFileOpen() 
{
	CFileDialog fDlg(TRUE, NULL, m_lastOpenedFileStr, myOpenFlags, myBackupFilter, NULL);
		
	fDlg.m_ofn.lpstrInitialDir = m_backupInitDirStr;
	fDlg.m_ofn.lpstrTitle = myOpenBackupFileTitle;

	if(IDOK == fDlg.DoModal())
	{
		CString filePath = fDlg.GetPathName();
		m_lastOpenedFileStr = fDlg.GetFileName();
		m_backupInitDirStr = filePath.Left(filePath.GetLength() - m_lastOpenedFileStr.GetLength());

		if(IsModified()) // ask to save before opening a new file
		{
			CString pathName = GetPathName();
			CString fileName = pathName.Mid(pathName.ReverseFind('\\') + 1);
			CString aStr;
			aStr.Format(_T("%shas been modified!\nSave contents first?"), 
				        fileName.IsEmpty() ? _T("This Document ") : fileName + _T('\n'));
			UINT retVal = 0;

			if(IDYES == (retVal = ((CMainFrame *)AfxGetMainWnd())->MessageBox(aStr, myOpenBackupFileTitle, MB_YESNOCANCEL | MB_ICONQUESTION)))
			{
				if(pathName.IsEmpty())
				{
					OnMyFileSaveAs();
				}
				else
				{
					OnSaveDocument(pathName); // saves modified when another file opens 
				}
			}
			else if(retVal == IDCANCEL) // cancel both save and open
			{
				return;
			}
		}

		OnOpenDocument(filePath);
		SetPathName(filePath, TRUE); // add to MRU list
		CDicksBackupView * pEditView = (CDicksBackupView*)m_viewList.GetHead();
		pEditView->OnInitialUpdate(); // gets document length etc.
	}
	else
	{
		DWORD errVal = CommDlgExtendedError();

		if(errVal)
		{
			CString aStr;
			aStr.Format(_T("File open dialog error\nError code = 0X%X"), errVal);
			(CMainFrame *)AfxGetMainWnd()->MessageBox(aStr, myOpenBackupFileTitle, MB_ICONSTOP);
		}
	}
}

BOOL CDicksBackupDoc::OnNewDocument()
{
	// call base class to create new backup manager
	BOOL retVal = CDocument::OnNewDocument();

	if(retVal)
	{
		ClearItemData();
		SetTitle(noBackupTitle); // use in place of stock "Untitled"
	}

	return retVal;
}

void CDicksBackupDoc::OnDoBackup()
{
	CDicksBackupView * pEditView = (CDicksBackupView*)m_viewList.GetHead();

	UINT nRows = pEditView->GetRows();

	if(nRows > 0)
	{
		// if a new backup, save it first
		if(GetPathName().IsEmpty())
		{
			OnMyFileSaveAs(); 
		}

		// path of backup document must exist
		if(!GetPathName().IsEmpty())
		{
			// start out with latest data
			OnEditRefresh();
			// count actions
			UINT nBackedUp = 0,
				 nCurrent = 0,
				 nProblem = 0,
				 nSkipped = 0;

			 // seprate stack frame for procress control so it destructed before next MssageBox
			{
				CWaitCursor aWait;
				// display progress in status bar
				PROGRESS(nRows, 1, _T(" Backing up files"));

				// work on rows from top down in list ctrl
				for(UINT i = 0; i < nRows; i++)
				{
					// data for row we are working on
					CItemData * pItemData = (CItemData *)pEditView->m_pListCtrl->GetItemData(i);

					switch (pItemData->m_lineFgColor)
					{
						case BACKED_UP_LIST_COLOR :

							nCurrent++;  
							break;

						case NOT_BACKED_UP_LIST_COLOR :

							// backup all or just selected files
							if(!m_bBackupSelected || LVIS_SELECTED == pEditView->m_pListCtrl->GetItemState(i, LVIS_SELECTED))
							{
								// last arg FALSE means overwrite if necessary
								if(::CopyFile(pItemData->m_sourcePathStr, pItemData->m_archivePathStr, FALSE)) 
								{
									nBackedUp++;
									pItemData->Refresh(); // recompose itemdata
								}
								else
								{
									pItemData->m_lineFgColor = PROBLEM_LIST_COLOR;
									pItemData->m_lineBkColor = PROBLEM_LIST_BKGND_COLOR;
									pItemData->m_nlastError = GetLastError();
									nProblem++;
								}

								// update on screen
								pEditView->UpdateRow(i);
								pEditView->m_pListCtrl->EnsureVisible(i, FALSE);
							}
							else
							{
								nSkipped++;
							}

							break;

						case PROBLEM_LIST_COLOR :

							nProblem++;
							break;
					}

					// remove any selection hilighting
					pEditView->m_pListCtrl->SetItemState(i, 0, LVIS_SELECTED);

					aProgerssCtrl.StepIt();
				}

				aProgerssCtrl.SetPos(0);
			}

			// ensure ready message in status bar
			READY_MSG();

			// save manager to disk
			if(nBackedUp > 0)
			{
				OnMyFileSave();
			}

			if(!m_bSilent) // silent when auto back triggers a backup
			{
				// Report on backup, use proper English 
				static char const procStr[] = _T("Report"),
			  				 			f[] = (_T("File was")),  
									   fs[] = (_T("Files were")), 
									   fh[] = (_T("File has")), 
									  fhs[] = (_T("Files have"));

				static char const fmtStr[] = _T("%d  %s up to date.\n\n")
											 _T("%d  %s backed up.\n\n")
											 _T("%d  %s skipped.\n\n")
											 _T("%d  %s problems. (see Results)\n")
											 _T("------------------------------------------\n")
											 _T("%d  total files");
				CString aStr;

				aStr.Format(fmtStr,
							nCurrent,  nCurrent  == 1 ? f : fs,
							nBackedUp, nBackedUp == 1 ? f : fs,
							nSkipped,  nSkipped  == 1 ? f : fs,
							nProblem,  nProblem  == 1 ? fh : fhs,
							nRows,     nRows ? f : fs);

				pEditView->MessageBox(aStr, procStr, MB_ICONINFORMATION);
			}
		}
	}
}

// must have files to backup but manager not saved yet is ok
void CDicksBackupDoc::OnUpdateDoBackup(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(((CDicksBackupView*)m_viewList.GetHead())->GetRows());	
}

void CDicksBackupDoc::OnEditRefresh() 
{
	CDicksBackupView * pEditView = (CDicksBackupView*)m_viewList.GetHead();

	UINT nRows = pEditView->GetRows();

	if(nRows)
	{
		PROGRESS(nRows, 1, _T(" Checking file status")); 

		for(UINT i = 0; i < nRows; i++)
		{
			if(!m_bBackupSelected || LVIS_SELECTED == pEditView->m_pListCtrl->GetItemState(i, LVIS_SELECTED))
			{
				CItemData * pItemData = (CItemData *)pEditView->m_pListCtrl->GetItemData(i);

				pItemData->Refresh();
				pEditView->UpdateRow(i);
			}

			aProgerssCtrl.StepIt();
		}

		READY_MSG();
	}
}

void CDicksBackupDoc::OnUpdateEditRefresh(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(((CDicksBackupView*)m_viewList.GetHead())->GetRows());	
}

void CDicksBackupDoc::OnMyFileCreate() 
{
	AfxGetMainWnd()->SendMessage(WM_COMMAND, ID_FILE_NEW);
	OnMyFileSaveAs();	
}

void CDicksBackupDoc::OnMyFileClose() 
{
	AfxGetMainWnd()->SendMessage(WM_COMMAND, ID_FILE_NEW);	
}

void CDicksBackupDoc::OnUpdateMyFileClose(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!GetPathName().IsEmpty());	
}

void CDicksBackupDoc::OnEditBackupSel() 
{
	m_bBackupSelected = TRUE;
	OnDoBackup();	
	m_bBackupSelected = FALSE;
}

void CDicksBackupDoc::OnUpdateEditBackupSel(CCmdUI* pCmdUI) 
{
	CDicksBackupView * pEditView = (CDicksBackupView*)m_viewList.GetHead();
	pCmdUI->Enable(pEditView->m_pListCtrl->GetSelectedCount() > 0);		
}

// append another backup to this one
void CDicksBackupDoc::OnEditAppend() 
{
	CFileDialog fDlg(TRUE, NULL, m_lastOpenedFileStr, myOpenFlags, myBackupFilter, NULL);
		
	fDlg.m_ofn.lpstrInitialDir = m_backupInitDirStr;
	fDlg.m_ofn.lpstrTitle = myImportBackupFileTitle;

	if(IDOK == fDlg.DoModal())
	{
		CString filePath = fDlg.GetPathName();

		if(GetPathName().CompareNoCase(filePath) != 0)
		{
			CFile aFile;

			if(aFile.Open(filePath, CFile::modeRead))
			{
				CArchive ar(&aFile, CArchive::load);
				CString sourcePathStr, archivePathStr, notesStr, verStr; 
				UINT nCount;

				ar >> verStr;

				if(verStr.Compare(versionString) == 0)
				{
					ar >> nCount; // count of following groups of 3 CStrings

					if(nCount > 0)
					{
						// display progress in status bar
						PROGRESS(nCount, 1, _T(" Locating Files, Please Wait."));

						// retrieve source path, archive path, and notes
						for(UINT i = 0; i < nCount; i++)
						{
							ar >> sourcePathStr;
							ar >> archivePathStr;
							ar >> notesStr;

							CItemData * pItemData = new CItemData(sourcePathStr, archivePathStr, notesStr);
							m_itemDataPtrArray.Add(pItemData);
							((CDicksBackupView*)m_viewList.GetHead())->AppendItemToList(pItemData);

							aProgerssCtrl.StepIt();
						}

						SetModifiedFlag(TRUE); 

						m_lastOpenedFileStr = fDlg.GetFileName();
						m_backupInitDirStr = filePath.Left(filePath.GetLength() - m_lastOpenedFileStr.GetLength());
					}

					READY_MSG();
				}
				else
				{
					AfxGetMainWnd()->MessageBox(_T("This is not a Backup file!"), myImportBackupFileTitle, MB_ICONWARNING);
				}

				ar.Close();
				aFile.Close();
			}
			else
			{
				CString aStr;
				aStr.Format(_T("Can't open file\n\n%s\n\n%s"), fDlg.GetFileTitle(), GetLastErrorMessage());
				(CMainFrame *)AfxGetMainWnd()->MessageBox(aStr, myImportBackupFileTitle, MB_ICONSTOP);
			}
		}
		else
		{
			(CMainFrame *)AfxGetMainWnd()->MessageBox(_T("Can't import backup to itself"), 
				                                      myImportBackupFileTitle, MB_ICONWARNING);
		}
	}
	else
	{
		DWORD errVal = CommDlgExtendedError();

		if(errVal)
		{
			CString aStr;
			aStr.Format(_T("File open dialog error\nError code = 0X%X"), errVal);
			(CMainFrame *)AfxGetMainWnd()->MessageBox(aStr, myImportBackupFileTitle, MB_ICONSTOP);
		}
	}
}

